segment = function(x, y, beta, p=0.25, maxseg, maxk, homthr) {
  if(!is.numeric(x)||!is.vector(x))
    stop("'x' (LRR) must be a numeric vector.")
  if(!is.numeric(y)||!is.vector(y))
    stop("'y' (BAF) must be a numeric vector.")
  
  x = matrix(x, ncol=1)
  y = matrix(y, ncol=1)
  n = nrow(x)
  maxseg = as.integer(maxseg)
  maxk  = as.integer(maxk)
  if(!((length(maxseg)==1) && !is.na(maxseg) && (maxseg<=n)))
    stop(sprintf("maxseg must be an integer of length 1 between 1 and nrow(x)=%d", n))
  if(!((length(maxk)==1) && !is.na(maxk) && (maxk<=n)))
    stop(sprintf("maxk must be an integer of length 1 between 1 and nrow(x)=%d", n))
  verbose = as.integer(0)
  
  G = costMatrix(x, y, beta, p, maxk, homthr)
  fs = .Call("findsegments", G, maxseg, verbose, PACKAGE="CnaStruct")

  bp = vector(mode="list", length=nrow(fs$th))
  for(i in seq(along=bp)) {
    stopifnot( fs$th[i, i] == n+1 )
    bp[[i]] = cbind(estimate=fs$th[i, 0:(i-1)])
  }
  
  new("segmentation",
    y = y,
    x = numeric(0), flag=integer(0),
    breakpoints = bp,
    logLik = fs$J,
    hasConfint = rep(FALSE, length(bp)))
} ## segment


breakpoints = function(x, y, beta=1, p=0.25, s=1, maxseg, maxk, homthr=0.95) {
	segm = segment(x, y, beta, p, maxseg, maxk, homthr)
	bestll = which.max(logLik(segm)-(1:length(logLik(segm)))*log(length(x))*s)
	# loglik = -(double)n/2.0*(1+log(2*M_PI)+log(z/n))
	# bestll = which.max(loglik-(1:length(logLik(segm)))*log(length(y))*s)
	# print(bestll)
	as.vector(segm@breakpoints[[bestll]])
}

setMethod("logLik", "segmentation", 
  function(object, penalty="none", ...) {
    ll   = object@logLik
    npar = 2*(1:length(ll))
    ndat = length(object@y)
    switch(penalty,
           none = ll,
           AIC = ll - npar,
           BIC = ll - log(ndat)*npar/2,
           stop(sprintf("Invalid value '%s' for argument 'penalty'.", penalty)))

  })

